@@ -24,25 +24,30 @@
 # SOFTWARE.
 import os
 import ast
+import datetime
 import json
 import copy
 import threading
 from collections import defaultdict
 from typing import Dict, Optional, List, Tuple, Set, Iterable, NamedTuple, Sequence, TYPE_CHECKING, Union
 import binascii
+import time
+
+import attr
 
 from . import util, bitcoin
 from .util import profiler, WalletFileException, multisig_type, TxMinedInfo, bfh
-from .invoices import Invoice
+from .invoices import Invoice, Request
 from .keystore import bip44_derivation
 from .transaction import Transaction, TxOutpoint, tx_from_any, PartialTransaction, PartialTxOutput
 from .logging import Logger
 from .lnutil import LOCAL, REMOTE, FeeUpdate, UpdateAddHtlc, LocalConfig, RemoteConfig, ChannelType
 from .lnutil import ImportedChannelBackupStorage, OnchainChannelBackupStorage
 from .lnutil import ChannelConstraints, Outpoint, ShachainElement
-from .json_db import StoredDict, JsonDB, locked, modifier
+from .json_db import StoredDict, JsonDB, locked, modifier, StoredObject
 from .plugin import run_hook, plugin_loaders
 from .submarine_swaps import SwapData
+from .version import ELECTRUM_VERSION
 
 if TYPE_CHECKING:
     from .storage import WalletStorage
@@ -52,7 +57,7 @@ if TYPE_CHECKING:
 
 OLD_SEED_VERSION = 4        # electrum versions < 2.0
 NEW_SEED_VERSION = 11       # electrum versions >= 2.0
-FINAL_SEED_VERSION = 49     # electrum >= 2.7 will set this to prevent
+FINAL_SEED_VERSION = 51     # electrum >= 2.7 will set this to prevent
                             # old versions from overwriting new format
 
 
@@ -62,6 +67,20 @@ class TxFeesValue(NamedTuple):
     num_inputs: Optional[int] = None
 
 
+@attr.s
+class DBMetadata(StoredObject):
+    creation_timestamp = attr.ib(default=None, type=int)
+    first_electrum_version_used = attr.ib(default=None, type=str)
+
+    def to_str(self) -> str:
+        ts = self.creation_timestamp
+        ver = self.first_electrum_version_used
+        if ts is None or ver is None:
+            return "unknown"
+        date_str = datetime.date.fromtimestamp(ts).isoformat()
+        return f"using {ver}, on {date_str}"
+
+
 class WalletDB(JsonDB):
 
     def __init__(self, raw, *, manual_upgrades: bool):
@@ -73,12 +92,13 @@ class WalletDB(JsonDB):
             self.load_plugins()
         else:  # creating new db
             self.put('seed_version', FINAL_SEED_VERSION)
+            self._add_db_creation_metadata()
             self._after_upgrade_tasks()
 
     def load_data(self, s):
         try:
             self.data = json.loads(s)
-        except:
+        except Exception:
             try:
                 d = ast.literal_eval(s)
                 labels = d.get('labels', {})
@@ -89,7 +109,7 @@ class WalletDB(JsonDB):
                 try:
                     json.dumps(key)
                     json.dumps(value)
-                except:
+                except Exception:
                     self.logger.info(f'Failed to convert label to json format: {key}')
                     continue
                 self.data[key] = value
@@ -198,6 +218,8 @@ class WalletDB(JsonDB):
         self._convert_version_47()
         self._convert_version_48()
         self._convert_version_49()
+        self._convert_version_50()
+        self._convert_version_51()
         self.put('seed_version', FINAL_SEED_VERSION)  # just to be sure
 
         self._after_upgrade_tasks()
@@ -904,17 +926,13 @@ class WalletDB(JsonDB):
                 }
         self.data['seed_version'] = 45
 
-    def _convert_version_46(self):
-        from .crypto import sha256d
-        if not self._is_upgrade_method_needed(45, 45):
-            return
+    def _convert_invoices_keys(self, invoices):
         # recalc keys of outgoing on-chain invoices
+        from .crypto import sha256d
         def get_id_from_onchain_outputs(raw_outputs, timestamp):
             outputs = [PartialTxOutput.from_legacy_tuple(*output) for output in raw_outputs]
             outputs_str = "\n".join(f"{txout.scriptpubkey.hex()}, {txout.value}" for txout in outputs)
             return sha256d(outputs_str + "%d" % timestamp).hex()[0:10]
-
-        invoices = self.data.get('invoices', {})
         for key, item in list(invoices.items()):
             is_lightning = item['lightning_invoice'] is not None
             if is_lightning:
@@ -926,6 +944,12 @@ class WalletDB(JsonDB):
             if newkey != key:
                 invoices[newkey] = item
                 del invoices[key]
+
+    def _convert_version_46(self):
+        if not self._is_upgrade_method_needed(45, 45):
+            return
+        invoices = self.data.get('invoices', {})
+        self._convert_invoices_keys(invoices)
         self.data['seed_version'] = 46
 
     def _convert_version_47(self):
@@ -970,6 +994,28 @@ class WalletDB(JsonDB):
             )
         self.data['seed_version'] = 49
 
+    def _convert_version_50(self):
+        if not self._is_upgrade_method_needed(49, 49):
+            return
+        requests = self.data.get('payment_requests', {})
+        self._convert_invoices_keys(requests)
+        self.data['seed_version'] = 50
+
+    def _convert_version_51(self):
+        from .lnaddr import lndecode
+        if not self._is_upgrade_method_needed(50, 50):
+            return
+        requests = self.data.get('payment_requests', {})
+        for key, item in list(requests.items()):
+            lightning_invoice = item.pop('lightning_invoice')
+            if lightning_invoice is None:
+                payment_hash = None
+            else:
+                lnaddr = lndecode(lightning_invoice)
+                payment_hash = lnaddr.paymenthash.hex()
+            item['payment_hash'] = payment_hash
+        self.data['seed_version'] = 51
+
     def _convert_imported(self):
         if not self._is_upgrade_method_needed(0, 13):
             return
@@ -1048,6 +1094,19 @@ class WalletDB(JsonDB):
                 msg += "\nPlease open this file with Electrum 1.9.8, and move your coins to a new wallet."
         raise WalletFileException(msg)
 
+    def _add_db_creation_metadata(self):
+        # store this for debugging purposes
+        v = DBMetadata(
+            creation_timestamp=int(time.time()),
+            first_electrum_version_used=ELECTRUM_VERSION,
+        )
+        assert self.get("db_metadata", None) is None
+        self.put("db_metadata", v)
+
+    def get_db_metadata(self) -> Optional[DBMetadata]:
+        # field only present for wallet files created with ver 4.4.0 or later
+        return self.get("db_metadata")
+
     @locked
     def get_txi_addresses(self, tx_hash: str) -> List[str]:
         """Returns list of is_mine addresses that appear as inputs in tx."""
@@ -1327,14 +1386,6 @@ class WalletDB(JsonDB):
         assert isinstance(txid, str)
         self.tx_fees.pop(txid, None)
 
-    @locked
-    def get_dict(self, name) -> dict:
-        # Warning: interacts un-intuitively with 'put': certain parts
-        # of 'data' will have pointers saved as separate variables.
-        if name not in self.data:
-            self.data[name] = {}
-        return self.data[name]
-
     @locked
     def num_change_addresses(self) -> int:
         return len(self.change_addresses)
@@ -1458,7 +1509,7 @@ class WalletDB(JsonDB):
         if key == 'invoices':
             v = dict((k, Invoice(**x)) for k, x in v.items())
         if key == 'payment_requests':
-            v = dict((k, Invoice(**x)) for k, x in v.items())
+            v = dict((k, Request(**x)) for k, x in v.items())
         elif key == 'adds':
             v = dict((k, UpdateAddHtlc.from_tuple(*x)) for k, x in v.items())
         elif key == 'fee_updates':
@@ -1499,6 +1550,8 @@ class WalletDB(JsonDB):
             v = Outpoint(**v)
         elif key == 'channel_type':
             v = ChannelType(v)
+        elif key == 'db_metadata':
+            v = DBMetadata(**v)
         return v
 
     def _should_convert_to_stored_dict(self, key) -> bool:
